@@ -18,6 +18,8 @@
 
 #include "config.h"
 
+#include <glib/gi18n-lib.h>
+
 #include <libsoup/soup.h>
 
 #include "goahttpclient.h"
@@ -80,8 +82,6 @@ http_client_check_data_free (gpointer user_data)
 {
   CheckData *data = user_data;
 
-  g_simple_async_result_complete_in_idle (data->res);
-
   if (data->cancellable_id > 0)
     {
       g_cancellable_disconnect (data->cancellable, data->cancellable_id);
@@ -152,8 +152,6 @@ http_client_check_response_cb (SoupSession *session, SoupMessage *msg, gpointer
 {
   GError *error;
   CheckData *data = user_data;
-  GMainContext *context;
-  GSource *source;
   gboolean op_res;
 
   error = NULL;
@@ -166,8 +164,11 @@ http_client_check_response_cb (SoupSession *session, SoupMessage *msg, gpointer
     goto out;
   else if (msg->status_code != SOUP_STATUS_OK)
     {
-      g_warning ("goa_http_client_check() failed: %u — %s", msg->status_code, msg->reason_phrase);
-      goa_utils_set_error_soup (&error, msg);
+      g_set_error (&error,
+                   GOA_ERROR,
+                   GOA_ERROR_FAILED, /* TODO: more specific */
+                   _("Code: %u — Unexpected response from server"),
+                   msg->status_code);
       goto out;
     }
 
@@ -181,14 +182,8 @@ http_client_check_response_cb (SoupSession *session, SoupMessage *msg, gpointer
   if (error != NULL)
     g_simple_async_result_take_error (data->res, error);
 
-  source = g_idle_source_new ();
-  g_source_set_priority (source, G_PRIORITY_DEFAULT_IDLE);
-  g_source_set_callback (source, http_client_check_data_free, data, NULL);
-  g_source_set_name (source, "[goa] http_client_check_data_free");
-
-  context = g_main_context_get_thread_default ();
-  g_source_attach (source, context);
-  g_source_unref (source);
+  g_simple_async_result_complete_in_idle (data->res);
+  g_idle_add (http_client_check_data_free, data);
 }
 
 static void
